package xiangmuguanli

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/mdb"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/enterprise/src/guanlixitong"
	"shylinux.com/x/enterprise/src/xiangmuguanli/model"
)

type Table struct {
	guanlixitong.Table
	list     string `name:"list project_uid uid auto" role:"void"`
	taskList string `name:"taskList" role:"worker"`
	doneList string `name:"doneList" role:"worker"`
}

func (s Table) Inputs(m *ice.Message, arg ...string) {
	switch arg[0] {
	case model.USER_PROJECT_ROLE:
		s.InputsListRole(m, UserProjectRoleList, arg...)
	case model.PROJECT_TYPE:
		s.InputsList(m, ProjectTypeList, arg...)
	default:
		s.Table.Inputs(m, arg...)
	}
}
func (s Table) RewriteAppend(m *ice.Message, arg ...string) *ice.Message {
	m.RewriteAppend(func(value, key string, index int) string {
		switch key {
		case model.USER_PROJECT_ROLE:
			value = UserProjectRole(kit.Int(value)).String()
		case model.PROJECT_TYPE:
			value = ProjectType(kit.Int(value)).String()
		case model.PLAN_STATUS:
			value = PlanStatus(kit.Int(value)).String()
		case model.TASK_STATUS:
			value = TaskStatus(kit.Int(value)).String()
		case model.DONE_STATUS:
			value = DoneStatus(kit.Int(value)).String()
		}
		return value
	})
	return s.Table.RewriteAppend(m)
}
func (s Table) SelectJoinPlan(m *ice.Message, arg ...string) {
	s.SelectJoin(m, Plan{}, model.TITLE, model.STATUS)
}
func (s Table) OtherCreate(m *ice.Message, target ice.Any, arg ...string) {
	m.Cmdy(target, s.Create, m.CommandKey()+"_uid", m.Option(model.UID), arg)
	kit.If(!m.IsErr(), func() { m.ProcessField(target, []string{m.Option(model.PROJECT_UID), m.Result()}) })
}
func (s Table) OtherList(m *ice.Message, target ice.Any, arg ...string) *ice.Message {
	m.Cmdy(target, s.Select, m.OptionSimple(model.PROJECT_UID), m.CommandKey()+"_uid", m.Option(model.UID)).Option("_command", ice.GetTypeKey(target))
	s.SelectJoinUser(m)
	return m
}
func (s Table) finishCheck(m *ice.Message, target ice.Any, name string, arg ...string) bool {
	count := m.Cmd(target, s.Select, m.CommandKey()+"_uid = ? AND status != ? AND status != ? AND status != ?",
		m.Option(model.UID), TaskRejected, TaskFinish, TaskCancel).Length()
	if m.WarnNotValid(count > 0, kit.Format("还有 %v 个未完成的%s", count, name)) {
		return true
	}
	return false
}
func (s Table) ChangeStatus(m *ice.Message, from, to int, arg ...string) {
	s.Table.ChangeStatus(m, m.Option(model.PROJECT_UID), m.Option(model.UID), from, to, arg...)
}
func (s Table) addCount(m *ice.Message, target ice.Any) Table {
	m.Cmd(target, s.AddCount, m.CommandKey()+"_count", kit.Select("-1", "1", m.ActionKey() == mdb.CREATE), m.Option(model.UID))
	return s
}
func (s Table) planCount(m *ice.Message) Table { return s.addCount(m, Plan{}) }
func (s Table) taskCount(m *ice.Message) Table { return s.addCount(m, Task{}) }

type Tables struct{ Table }

func (s Tables) BeforeMigrate(m *ice.Message, arg ...string) {}
