package xiangmuguanli

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/enterprise/src/xiangmuguanli/model"
)

type Done struct {
	Table
	order   string `data:"3"`
	fields  string `data:"title,content,done_status,begin_time,end_time,process_time,finish_time,task_uid,plan_uid,user_uid"`
	create  string `name:"create task_uid* title* content* begin_time:select@date end_time:select@date" role:"worker"`
	modify  string `name:"modify title* content* begin_time:select@date end_time:select@date" role:"worker"`
	remove  string `name:"remove" role:"worker"`
	process string `name:"process" role:"worker"`
	finish  string `name:"finish" role:"worker"`
}

func (s Done) Create(m *ice.Message, arg ...string) {
	s.ValueCreate(m, kit.ArgDef(arg, kit.Simple(model.BEGIN_TIME, m.Time(), model.END_TIME, m.Time("72h"))...)...)
	s.SendMessage(m, "", "")
	s.taskCount(m).DashboardUpdate(m)
}
func (s Done) Remove(m *ice.Message, arg ...string) {
	s.ValueRemove(m, arg...)
	s.taskCount(m).DashboardUpdate(m)
}
func (s Done) List(m *ice.Message, arg ...string) {
	user_uid := m.Option(model.USER_UID)
	s.ValueList(m, arg).Table(func(value ice.Maps) {
		button := []ice.Any{}
		defer func() { m.PushButton(button...) }()
		switch DoneStatus(kit.Int(value[model.DONE_STATUS])) {
		case DoneCreate:
			if value[model.USER_UID] == user_uid {
				button = append(button, s.Process)
			}
		case DoneProcess:
			if value[model.USER_UID] == user_uid {
				button = append(button, s.Finish)
			}
		case DoneFinish:
			kit.If(m.FieldsIsDetail(), func() { s.DoneMessage(m) })
		}
	}).Display("")
	s.SelectJoinPlan(m)
}
func (s Done) Process(m *ice.Message, arg ...string) {
	s.changeStatus(m, DoneCreate, DoneProcess)
}
func (s Done) Finish(m *ice.Message, arg ...string) {
	s.changeStatus(m, DoneProcess, DoneFinish)
}

func init() { ice.TeamCtxCmd(Done{}) }

func (s Done) changeStatus(m *ice.Message, from, to DoneStatus) {
	s.ChangeStatus(m, int(from), int(to), m.ActionKey()+"_time", m.Time())
}

type DoneStatus int

const (
	DoneCreate DoneStatus = iota
	DoneProcess
	DoneFinish
)

var DoneStatusList = map[DoneStatus]string{
	DoneCreate:  "create",
	DoneProcess: "process",
	DoneFinish:  "finish",
}

func (s DoneStatus) String() string { return DoneStatusList[s] }
