package xiangmuguanli

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/enterprise/src/xiangmuguanli/model"
)

type Task struct {
	Table
	order      string `data:"2"`
	fields     string `data:"title,content,task_status,done_count,begin_time,end_time,process_time,finish_time,plan_uid,user_uid"`
	create     string `name:"create plan_uid* title* content* begin_time:select@date end_time:select@date" role:"worker"`
	modify     string `name:"modify title* content* begin_time:select@date end_time:select@date" role:"worker"`
	remove     string `name:"remove" role:"worker"`
	reject     string `name:"reject" role:"leader"`
	approve    string `name:"approve" role:"leader"`
	process    string `name:"process" role:"worker"`
	submit     string `name:"submit" role:"worker"`
	reback     string `name:"reback" role:"leader"`
	finish     string `name:"finish" role:"leader"`
	cancel     string `name:"cancel" role:"leader"`
	doneCreate string `name:"doneCreate title* content* begin_time:select@date end_time:select@date" role:"worker"`
}

func (s Task) Create(m *ice.Message, arg ...string) {
	s.ValueCreate(m, kit.ArgDef(arg, kit.Simple(model.BEGIN_TIME, m.Time(), model.END_TIME, m.Time("72h"))...)...)
	s.SendMessage(m, "", "")
	s.planCount(m).DashboardUpdate(m)
}
func (s Task) Remove(m *ice.Message, arg ...string) {
	s.ValueRemove(m, arg...)
	s.planCount(m).DashboardUpdate(m)
}
func (s Task) List(m *ice.Message, arg ...string) {
	if !s.IsWorker(m) {
		s.ApplyCheck(m, arg...)
		return
	}
	user_uid := m.Option(model.USER_UID)
	isLeader := s.IsLeader(m)
	isWorker := s.IsWorker(m)
	s.Orders(m, model.STATUS, model.DONE_COUNT, s.Desc(model.CREATED_AT))
	s.ValueList(m, arg).Table(func(value ice.Maps) {
		button := []ice.Any{}
		defer func() { m.PushButton(button...) }()
		switch TaskStatus(kit.Int(value[model.TASK_STATUS])) {
		case TaskCreate:
			if isLeader {
				button = append(button, s.Approve, s.Reject)
			}
			if value[model.USER_UID] == user_uid {
				button = append(button, s.Modify, s.Remove)
			}
		case TaskApproved:
			if value[model.USER_UID] == user_uid {
				button = append(button, s.Process)
			}
		case TaskProcess:
			if isWorker {
				button = append(button, s.DoneCreate)
			}
			if value[model.USER_UID] == user_uid {
				button = append(button, s.Submit)
			}
		case TaskSubmit:
			if isLeader {
				button = append(button, s.Finish, s.Reback, s.Cancel)
			}
		case TaskFinish:
			kit.If(m.FieldsIsDetail(), func() { s.DoneMessage(m) })
		}
	}).Display("")
	s.SelectJoinPlan(m)
	m.Option("otherList", "doneList")
}
func (s Task) Reject(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, int(TaskCreate), int(TaskRejected))
}
func (s Task) Approve(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, int(TaskCreate), int(TaskApproved))
}
func (s Task) Process(m *ice.Message, arg ...string) {
	s.changeStatus(m, TaskApproved, TaskProcess)
}
func (s Task) Submit(m *ice.Message, arg ...string) {
	if s.finishCheck(m, Done{}, "提报") {
		return
	}
	s.ChangeStatus(m, int(TaskProcess), int(TaskSubmit))
}
func (s Task) Reback(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, int(TaskSubmit), int(TaskProcess))
}
func (s Task) Finish(m *ice.Message, arg ...string) {
	s.changeStatus(m, TaskSubmit, TaskFinish)
}
func (s Task) Cancel(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, int(TaskSubmit), int(TaskCancel))
}
func (s Task) DoneCreate(m *ice.Message, arg ...string) {
	s.OtherCreate(m, Done{}, append(arg, m.OptionSimple(model.PLAN_UID)...)...)
}
func (s Task) DoneList(m *ice.Message, arg ...string) {
	s.OtherList(m, Done{}).Display("task.js")
}

func init() { ice.TeamCtxCmd(Task{}) }

func (s Task) changeStatus(m *ice.Message, from, to TaskStatus) {
	s.ChangeStatus(m, int(from), int(to), m.ActionKey()+"_time", m.Time())
}

type TaskStatus int

const (
	TaskCreate TaskStatus = iota
	TaskRejected
	TaskApproved
	TaskProcess
	TaskSubmit
	TaskReback
	TaskFinish
	TaskCancel
)

var TaskStatusList = map[TaskStatus]string{
	TaskCreate:   "create",
	TaskRejected: "rejected",
	TaskApproved: "approved",
	TaskProcess:  "process",
	TaskSubmit:   "submit",
	TaskReback:   "reback",
	TaskFinish:   "finish",
	TaskCancel:   "cancel",
}

func (s TaskStatus) String() string { return TaskStatusList[s] }
